/*-------------------------------------------------------------------
Name......... game-main.c
Author....... Rod Boyce
Date......... 01/10/04
Description.. Example code to demostrate the capibilities of the
............. Audio driver and Enhanced Video driver.
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
DECLARATION FORMATS
Type defs........... UPPER CASE
#defines............ UPPER CASE
Global Functions.... GlobalFunction();
Static Functions.... staticfunction();
Global Variables.... GlobalVariable;
Static Variables.... staticvariable;
Local Variables..... localvariable;
Structure Elements.. Structure.element;
-------------------------------------------------------------------*/

/****   Include Files */

#include "hardware.h"
#include "peripherals.h"
#include "interface.h"
#include "game.h"

#include "config.h"
#include "fpga.h"
#include "game-main.h"

/****	Defines ****/
/* Some stdio function require a prototype definition not to return warning */
int sprintf(char *, const char *, ...); /* <- from <stdio.h> */

/****   Global Variables */
/****	Local variables */
static tsSCREENS sScreen;

/* Screen Image names to load from compact flash */
static char *apcScreenImages[] =
{
    "vegas3_1.bmp",
    "pluto6_banner.bmp",
    "ball-red.bmp",
    "ball-seagreen.bmp"
};

/* Sound files to use from compact flash */
static char *apcSoundFiles[] =
{
    "spies-r-us1.wav",
    "left.wav",
    "right.wav",
    "mind-out.wav"
};

/* sprite infomation is loaded into these vars */
static BMPSPRITEINFO background;
static BMPSPRITEINFO advert;
static BMPSPRITEINFO sBall[ NUM_SPRITES ];
static BMPSPRITEINFO sCompName[ NUM_COMP_IMAGES ];

/* Structures to store the animation data files */
static ANIMATION sHeberAnimation;
static ANIMATION sRedBall;
static ANIMATION sGreenBall;

/* Wave files to play based on different events */
static WAVFILEINFO sLoopSound;
static WAVFILEINFO sLeftBounce;
static WAVFILEINFO sRightBounce;
static WAVFILEINFO sCollision;

static volatile UBYTE qLeftHit;
static volatile UBYTE qRightHit;
static volatile UBYTE qRedGreenHit;

/* Buffer address to save background data to for animation */
static ULONG alSaveBuffer[ 2 ];

static void gm_InitialiseScreen( void );
static void gm_InitialiseCompactFlash( void );
static UBYTE gm_LoadImages( void );
static BYTE gm_LoadWindowsBmp( char *acFileName, BMPFILEINFO *sBmpInfo, UWORD *pwTmpMem,
                               ULONG *plStartOfFreeMem );
static void gm_CheckForCollision( ANIMATION *psCurAnim, ANIMATION *psOthAnim );
static void gm_DummyAnimation( ANIMATION *sAnim );
static void gm_IsBallAtEdge( ANIMATION *anim );
static void gm_InitialiseSound( void );

/* Debug serial device */
static const DEVICE* pSerialDevice;

/*-------------------------------------------------------------------
Name......... gm_GameMain()
Author....... Rod Boyce
Date......... 01/10/04
Description.. This is the main game routing
............. This function will set-up the game by initisalising the
............. game data structures loading the graphics into video
............. memory and then starting the game.
Parameters... None
Return....... None
History...... 01/10/04 - First Pass  - RB
-------------------------------------------------------------------*/

void gm_GameMain( void )
{

    /* Set up debug UART */
    pSerialDevice = &SerialDevice;
	ConfigureSerial( pSerialDevice, 0, BAUD9600, BAUD9600,
                     BITS8, STOP1, PARITYNONE, TXNONE, RXNONE );

    TxString(pSerialDevice, 0,"\r\r Heber Advanced video demo program\r" );

    /* Compact flash card */
    gm_InitialiseCompactFlash();

    /* initialise screens size and load images from compact flash */
    gm_InitialiseScreen();

    /* initialise the sound system */
    gm_InitialiseSound();

    /* Load background images onto the base layer */
    CalypsoPutSprite( &Calypso32PlusDevice, BL_LAYER, DOUBLEBUFFER_AUTO, 0, 0, background.lStartAddr,
                      background.sBmp.imagewidth, background.sBmp.imageheight, NORMAL_MODE );
    CalypsoPutSprite( &Calypso32PlusDevice, BL_LAYER, DOUBLEBUFFER_AUTO, 0, 0, advert.lStartAddr,
                      advert.sBmp.imagewidth, advert.sBmp.imageheight, TRANSPARENT_MODE );

    /* Initialise 'Heber' animation at the top of the screen */
    CalypsoDefineAnimation( &Calypso32PlusDevice, &sHeberAnimation,
                            ML_LAYER,
                            ANIMATION_RUN_FOREVER,
                            ( SCREEN_HSIZE / 2 ) - ( sCompName[ 0 ].sBmp.imagewidth / 2 ), 0,
                            0, 0,
                            sCompName[ 0 ].lStartAddr,
                            sCompName[ 1 ].lStartAddr - sCompName[ 0 ].lStartAddr,
                            NUM_COMP_IMAGES, 0,
                            sCompName[ 0 ].sBmp.imagewidth,
                            sCompName[ 0 ].sBmp.imageheight,
                            ANIMATION_TYPE3,
                            TRANSPARENT_MODE,
                            alSaveBuffer[ 0 ],
                            alSaveBuffer[ 1 ],
                            gm_DummyAnimation );

    /* Initialise the red ball as an animation */
    CalypsoDefineAnimation( &Calypso32PlusDevice, &sRedBall, ML_LAYER, ANIMATION_RUN_FOREVER,
                            REAL_HSIZE * 2 / 3, 0, 5, 5,
                            sBall[ RED_BALL ].lStartAddr, 0, 1, 0,
                            sBall[ RED_BALL ].sBmp.imagewidth,
                            sBall[ RED_BALL ].sBmp.imageheight,
                            ANIMATION_TYPE3, TRANSPARENT_MODE,
                            sBall[ RED_BALL ].alSaveBuffer[ 0 ],
                            sBall[ RED_BALL ].alSaveBuffer[ 1 ],
                            gm_IsBallAtEdge );

    /* Initialise the green ball as an animation */
    CalypsoDefineAnimation( &Calypso32PlusDevice, &sGreenBall, ML_LAYER, ANIMATION_RUN_FOREVER,
                            0, 0, 7, 7,
                            sBall[ GREEN_BALL ].lStartAddr, 0, 1, 0,
                            sBall[ GREEN_BALL ].sBmp.imagewidth,
                            sBall[ GREEN_BALL ].sBmp.imageheight,
                            ANIMATION_TYPE3, TRANSPARENT_MODE,
                            sBall[ GREEN_BALL ].alSaveBuffer[ 0 ],
                            sBall[ GREEN_BALL ].alSaveBuffer[ 1 ],
                            gm_IsBallAtEdge );


    /* Add the animations to the animation list in the order they should be processed */
    CalypsoAddToAnimationList( &Calypso32PlusDevice, &sRedBall );
    CalypsoAddToAnimationList( &Calypso32PlusDevice, &sGreenBall );
    CalypsoAddToAnimationList( &Calypso32PlusDevice, &sHeberAnimation );

    /* Start the double buffer and animation system at 30Hz */
    CalypsoSetFrameRate( &Calypso32PlusDevice, FRAME_30HZ );

    /* Show the the animations */
    CalypsoStartAnimation( &Calypso32PlusDevice, &sHeberAnimation );
    CalypsoStartAnimation( &Calypso32PlusDevice, &sRedBall );
    CalypsoStartAnimation( &Calypso32PlusDevice, &sGreenBall );

    /* Ok everything is ready to go so let's start playing the looped sound */
    PlaybackCachedWavFile( AUDIO_STEREO0 | AUDIO_LOOPED, &sLoopSound );

    /* Start the main loop - This is very simple as all the animation is taken care of with
        the advanced video driver using the animation API */
    while( 1 )
    {
        Wait( 1 );

        /* If a ball hit the left side play the left bounce soune */
        if( 0 != qLeftHit )
        {
            if( 0 == SoundChannelBusy( AUDIO_LEFT1 ))
            {
                PlaybackCachedWavFile( AUDIO_LEFT1, &sLeftBounce );
            }
            qLeftHit = 0;
        }

        /* If a ball hits the right side play the right bounce sound */
        if( 0 != qRightHit )
        {
            if( 0 == SoundChannelBusy( AUDIO_RIGHT1 ))
            {
                PlaybackCachedWavFile( AUDIO_RIGHT1, &sRightBounce );
            }
            qRightHit = 0;
        }

        /* If the red and green balls hit each other play the collision sound */
        if( 0 != qRedGreenHit )
        {
            if( 0 == SoundChannelBusy( AUDIO_STEREO1 ))
            {
                StopAudioChannel( AUDIO_STEREO1 );
            }
            PlaybackCachedWavFile( AUDIO_STEREO1, &sCollision );
            qRedGreenHit = 0;
        }

    } /* While */
}

/*-------------------------------------------------------------------
Name......... gm_InitialiseCompactFlash()
Author....... Rod Boyce
Date......... 01/10/04
Description.. Initialise the compact flash device and mount the
............. file-system
Parameters... None
Return....... None
History...... 01/10/04 - First Pass  - RB
-------------------------------------------------------------------*/

static void gm_InitialiseCompactFlash( void )
{
	BYTE str[256];

    /* initialise compact flash card */
    TxString(pSerialDevice, 0, "Initialise Compact Flash Card\r\r");
    TxString(pSerialDevice, 0, "Identify Drive:\r");

    /* Initalise IDE drive */
    if( 0 == ReadDriveID( &ATADevice ))
    {

		sprintf(str, "\tModel:\t%s\r\t\tRev:\t%s\r\tSerial No:\t%s\r\tCylinders:\t%3d\r\tHeads:\t\t%3d\r\tSectors:\t%3d\r\r",
				GetDriveModel(&ATADevice),
				GetDriveRev(&ATADevice),
				GetDriveSerialno(&ATADevice),
				GetDriveCyls(&ATADevice),
				GetDriveHeads(&ATADevice),
				GetDriveSectors(&ATADevice));

		TxString(pSerialDevice,0,str);

       /* mount FAT32 File system */
        TxString(pSerialDevice, 0, "\rMounting FAT32 File System\r");
        if( 0 == MountFAT32( &ATADevice ))
        {
			TxString(pSerialDevice, 0, "\tPartition found on Primary Partition\r");
			sprintf(str, "\tVolume: %s Size: %ld MBytes \r\r",
					FAT32VolumeLabel( &ATADevice ), FAT32PartitionSize( &ATADevice ));
			TxString(pSerialDevice, 0, str );

        }
        else
        {
			TxString(pSerialDevice, 0, "\tFailed to mount drive\r");
        }
    }
    else
    {
		TxString(pSerialDevice, 0, "\tTimeout... \r");
    }
}

/*-------------------------------------------------------------------
Name......... gm_InitialiseScreen()
Author....... Rod Boyce
Date......... 01/10/04
Description.. Initialise the screen for game
Parameters... None
Return....... None
History...... 01/10/04 - First Pass  - RB
-------------------------------------------------------------------*/

static void gm_InitialiseScreen( void )
{
    /* Initialise the screen memory */
    CalypsoInitDisplayMode( &Calypso32PlusDevice, Calypso800x600 );
	CalypsoDisableLayer( &Calypso32PlusDevice, ALL_SELECT );

    /* Firstly reserve memory for all the needed screens */
    /* base layer screen 0 */
    sScreen.alScreen[ SCREEN_BACKGROUND ] = 0;

    /* Initiaalise the base layer screen */
    sScreen.alScreen[ SCREEN_MIDDLE ] =
        CalypsoInitLayer( &Calypso32PlusDevice, BL_LAYER, DIRECT_COLOR, SINGLEBUFFER, BLACK,
                          SCREEN_HSIZE, SCREEN_VSIZE, sScreen.alScreen[ SCREEN_BACKGROUND ]);

    /* Initialise the middle double buffered layer */
    sScreen.lScratchMemory =
        CalypsoInitLayer( &Calypso32PlusDevice, ML_LAYER, DIRECT_COLOR, DOUBLEBUFFER, BLACK,
                          SCREEN_HSIZE, SCREEN_VSIZE, sScreen.alScreen[ SCREEN_MIDDLE ]);
    sScreen.lScratchMemory += OFFSET_VIDEO_MEMORY;

    /* Initialise the non-display memory for bitmaps and bitmap masks */
    /* non displayed video memory just after screens. Offset 0x30000000 is added because variable
       will then be converted into a pointer in video memory */
    sScreen.lStartOfFreeMem = ( sScreen.lScratchMemory + ( SCREEN_HSIZE * SCREEN_VSIZE * 4 ));

    /* Clear and enable the display layers */
    CalypsoClearLayer( &Calypso32PlusDevice, BL_LAYER, DOUBLEBUFFER_AUTO );
    CalypsoClearLayer( &Calypso32PlusDevice,  ML_LAYER, DOUBLEBUFFER_FRAME0 );
    CalypsoClearLayer( &Calypso32PlusDevice,  ML_LAYER, DOUBLEBUFFER_FRAME1 );
    CalypsoEnableLayer( &Calypso32PlusDevice, BASE_ENABLE );
    CalypsoEnableLayer( &Calypso32PlusDevice, MIDDLE_ENABLE );
    CalypsoEnableDoubleBuffer(&Calypso32PlusDevice);

    Wait( 20 ); /* Wait a bit for screen ops to complete */

    /* load images from compact flash card into non displayed video memory */
    if( !gm_LoadImages())
    {
		TxString(pSerialDevice, 0, "Error load bmp file\r" );
    }
    else
    {
		TxString(pSerialDevice, 0, "Images loaded\r" );
    }
}

/*-------------------------------------------------------------------
Name......... gm_LoadImages()
Author....... Rod Boyce
Date......... 01/10/04
Description.. Load all necessary images and mask into video memory.
Parameters... None
Return....... Return  zero if sucessful, positive value on failure
History...... 01/10/04 - First Pass  - RB
-------------------------------------------------------------------*/

static UBYTE gm_LoadImages( void )
{
    static char acTmp[ 15 ];
    UWORD *plScratchMemory;
    ULONG i;
    BYTE str[256];

    /* set access folder to correct path*/
    ChangeDir( &ATADevice, "transparencyfolder" );

    /* Address of the scratch memory used by the graphics module */
    plScratchMemory = (UWORD *)sScreen.lScratchMemory;

    /* Load the background image */
    background.lStartAddr = sScreen.lStartOfFreeMem;
    if( gm_LoadWindowsBmp( apcScreenImages[ 0 ], &background.sBmp, plScratchMemory,
                           &sScreen.lStartOfFreeMem ))
    {
		sprintf(str, "Failed to load %s\r",	apcScreenImages[0]);
		TxString(pSerialDevice, 0, str );
        return( FALSE );
    }
    else
    {
		sprintf(str, "%s loaded\r",	apcScreenImages[0]);
		TxString(pSerialDevice, 0, str );
    }

    /* Load advert and create mask for the advert */
    advert.lStartAddr = sScreen.lStartOfFreeMem;
    if( gm_LoadWindowsBmp( apcScreenImages[ 1 ], &advert.sBmp, plScratchMemory,
                           &sScreen.lStartOfFreeMem ))
    {
		sprintf(str, "Failed to load %s\r",	apcScreenImages[1]);
		TxString(pSerialDevice, 0, str );
        return FALSE;
    }
    else
    {
		sprintf(str, "%s loaded\r",	apcScreenImages[1]);
		TxString(pSerialDevice, 0, str );
    }

    advert.lMaskAddr = sScreen.lStartOfFreeMem;    /* start address of the mask */
    sScreen.lStartOfFreeMem = OFFSET_VIDEO_MEMORY;

    sScreen.lStartOfFreeMem +=
    	CalypsoCreateMask( &Calypso32PlusDevice,
						   ( const UWORD *)( advert.lStartAddr ),
                           advert.lMaskAddr - OFFSET_VIDEO_MEMORY,
                           advert.sBmp.imagewidth *
                           advert.sBmp.imageheight * BYTES_PER_PIXEL );

    /* red ball image */
    sBall[ RED_BALL ].lStartAddr = sScreen.lStartOfFreeMem;
    if( gm_LoadWindowsBmp( apcScreenImages[ 2 ], &sBall[ RED_BALL ].sBmp, plScratchMemory,
                           &sScreen.lStartOfFreeMem ))
    {
		sprintf(str, "Failed to load %s\r",	apcScreenImages[2]);
		TxString(pSerialDevice, 0, str );
        return( FALSE );
    }
    else
    {
		sprintf(str, "%s loaded\r",	apcScreenImages[2]);
		TxString(pSerialDevice, 0, str );
    }

    /* create a mask of the red ball */
    sBall[ RED_BALL ].lMaskAddr = sScreen.lStartOfFreeMem;
    sScreen.lStartOfFreeMem = OFFSET_VIDEO_MEMORY;

    sScreen.lStartOfFreeMem +=
        CalypsoCreateMask( &Calypso32PlusDevice,
				   ( const UWORD *)( sBall[ RED_BALL ].lStartAddr ),
                   sBall[ RED_BALL ].lMaskAddr - OFFSET_VIDEO_MEMORY,
                   sBall[ RED_BALL ].sBmp.imagewidth *
                   sBall[ RED_BALL ].sBmp.imageheight * BYTES_PER_PIXEL );

    sBall[ RED_BALL ].alSaveBuffer[ 0 ] = sScreen.lStartOfFreeMem;
    sScreen.lStartOfFreeMem += ( sBall[ RED_BALL ].sBmp.imagewidth *
                                 sBall[ RED_BALL ].sBmp.imageheight *
                                 BYTES_PER_PIXEL );
    sBall[ RED_BALL ].alSaveBuffer[ 1 ] = sScreen.lStartOfFreeMem;
    sScreen.lStartOfFreeMem += ( sBall[ RED_BALL ].sBmp.imagewidth *
                                 sBall[ RED_BALL ].sBmp.imageheight *
                                 BYTES_PER_PIXEL );


    /* Green ball image */
    sBall[ GREEN_BALL ].lStartAddr = sScreen.lStartOfFreeMem;
    if( gm_LoadWindowsBmp( apcScreenImages[ 3 ], &sBall[ GREEN_BALL ].sBmp, plScratchMemory,
                           &sScreen.lStartOfFreeMem ))
    {
		sprintf(str, "Failed to load %s\r",	apcScreenImages[3]);
		TxString(pSerialDevice, 0, str );
        return( FALSE );
    }
    else
    {
		sprintf(str, "%s loaded\r",	apcScreenImages[3]);
		TxString(pSerialDevice, 0, str );
    }

    /* create a mask of the Green ball */
    sBall[ GREEN_BALL ].lMaskAddr = sScreen.lStartOfFreeMem;
    sScreen.lStartOfFreeMem = OFFSET_VIDEO_MEMORY;

    sScreen.lStartOfFreeMem +=
        CalypsoCreateMask( &Calypso32PlusDevice,
				   ( const UWORD *)( sBall[ GREEN_BALL ].lStartAddr ),
                   sBall[ GREEN_BALL ].lMaskAddr - OFFSET_VIDEO_MEMORY,
                   sBall[ GREEN_BALL ].sBmp.imagewidth *
                   sBall[ GREEN_BALL ].sBmp.imageheight *
                   BYTES_PER_PIXEL );

    sBall[ GREEN_BALL ].alSaveBuffer[ 0 ] = sScreen.lStartOfFreeMem;
    sScreen.lStartOfFreeMem += ( sBall[ RED_BALL ].sBmp.imagewidth *
                                 sBall[ RED_BALL ].sBmp.imageheight *
                                 BYTES_PER_PIXEL );
    sBall[ GREEN_BALL ].alSaveBuffer[ 1 ] = sScreen.lStartOfFreeMem;
    sScreen.lStartOfFreeMem += ( sBall[ RED_BALL ].sBmp.imagewidth *
                                 sBall[ RED_BALL ].sBmp.imageheight *
                                 BYTES_PER_PIXEL );


    /* Read in animation frames */
    for( i = 0; i < NUM_COMP_IMAGES; i++ )
    {
        sprintf( acTmp, "heber%02ld.bmp", i + 1 );
        sCompName[ i ].lStartAddr = sScreen.lStartOfFreeMem;
        if( gm_LoadWindowsBmp( acTmp, &sCompName[ i ].sBmp, plScratchMemory,
                               &sScreen.lStartOfFreeMem ))
        {
			sprintf(str, "Failed to load %s\r",	acTmp);
			TxString(pSerialDevice, 0, str );
	        return( FALSE );
        }
        else
        {
			sprintf(str, "%s loaded\r",	acTmp);
			TxString(pSerialDevice, 0, str );
        }

        /* Create mask for each frame */
        sCompName[ i ].lMaskAddr = sScreen.lStartOfFreeMem;
        sScreen.lStartOfFreeMem = OFFSET_VIDEO_MEMORY;

        sScreen.lStartOfFreeMem +=
        	CalypsoCreateMask( &Calypso32PlusDevice,
							  ( const UWORD *)( sCompName[ i ].lStartAddr ),
							  sCompName[ i ].lMaskAddr - OFFSET_VIDEO_MEMORY,
							  sCompName[ i ].sBmp.imagewidth *
							  sCompName[ i ].sBmp.imageheight * BYTES_PER_PIXEL);
    }

    alSaveBuffer[ 0 ] = sScreen.lStartOfFreeMem;
    sScreen.lStartOfFreeMem += sCompName[ 0 ].sBmp.imagewidth *
        sCompName[ 0 ].sBmp.imageheight * BYTES_PER_PIXEL;

    alSaveBuffer[ 1 ] = sScreen.lStartOfFreeMem;
    sScreen.lStartOfFreeMem += sCompName[ 0 ].sBmp.imagewidth *
        sCompName[ 0 ].sBmp.imageheight * BYTES_PER_PIXEL;

    return( TRUE );
}

/*-------------------------------------------------------------------
Name......... gm_LoadWindowsBmp()
Author....... Rod Boyce
Date......... 01/10/04
Description.. Load windows BMP Image and calculate next free memory
............. address.
Parameters... 1) acFileName - Pointer to filename
............. 2) psBmpInfo - Pointer to bmp info structure
............. 3) pwTempMem - Pointer to scratch memory info structure
............. 4) plStartOfFreeMem - Pointer to start of free memory
Return....... Return  zero if sucessful, positive value on failure
History...... 01/10/04 - First Pass  - RB
-------------------------------------------------------------------*/

static BYTE gm_LoadWindowsBmp( BYTE *acFileName, BMPFILEINFO *psBmpInfo,
							   UWORD *pwTmpMem, ULONG *plStartOfFreeMem )
{
    ULONG lFreeMem;
    UBYTE bRtnv = 0xff; /* failed until passed */

    /* Get free memory address */
    lFreeMem = *plStartOfFreeMem;

    bRtnv = LoadMsWinBmp24( &ATADevice, acFileName, psBmpInfo, pwTmpMem, lFreeMem );
    if( 0 == bRtnv )
    {
        /* File found and loaded */

        /* Calculate new start of free memory address */
        lFreeMem = ( lFreeMem + ( psBmpInfo->imagewidth * psBmpInfo->imageheight *
                                  BYTES_PER_PIXEL ));

        /* Update Start of free memory */
        *plStartOfFreeMem = lFreeMem;
    }

    return( bRtnv );
}

/*-------------------------------------------------------------------
Name......... gm_CheckForCollision()
Author....... Rod Boyce
Date......... 01/10/04
Description.. Check the position of both balls on the screen for a
............. collision.
............. Check the position of both balls on the screen and see
............. if a collision has occured.  This is easy as both balls
............. are round so if the distance of both balls is less than
............. the diameter on one ball a collision has occured.
............. Note:   This does not take into account the velocity
.............         that the balls are traveling in.  The outcome
.............         of this is that you see some very weird effects.
Parameters... 1) psCurAnim - pointer to the animation to action the
.............    collision on
............. 2) psOthAnim - pointer to the other animation to check
.............    against
Return....... None
History...... 01/10/04 - First Pass  - RB
-------------------------------------------------------------------*/

static void gm_CheckForCollision( ANIMATION *psCurAnim, ANIMATION *psOthAnim )
{
    LONG zDeltaX;
    LONG zDeltaY;
    LONG diameter;

    zDeltaX = psCurAnim->x - psOthAnim->x;
    zDeltaY = psCurAnim->y - psOthAnim->y;

    /* diameter = ( sRedBall.sBmp.imagewidth - 2 ) * 9 / 10; */
    diameter = 60; /* Cheating but there you go it is simple and fast */
    if(( abs( zDeltaX ) < diameter ) && ( abs( zDeltaY ) < diameter ))
    {
        qRedGreenHit = 1;
        if(( zDeltaX > 0 ) && ( zDeltaY > 0 ))
        {
            /* Q 1 */
            if( abs( zDeltaX ) < abs( zDeltaY ))
            {
                psCurAnim->dx = 0 - psCurAnim->dx;
            }
            else
            {
                psCurAnim->dy = 0 - psCurAnim->dy;
            }
        }
        else if(( zDeltaX < 0 ) && ( zDeltaY > 0 ))
        {
            /* Q 2 */
            if( abs( zDeltaX ) < abs( zDeltaY ))
            {
                psCurAnim->dy = 0 - psCurAnim->dy;
            }
            else
            {
                psCurAnim->dy = 0 - psCurAnim->dy;
                psCurAnim->dx = 0 - psCurAnim->dx;
            }
        }
        else if(( zDeltaX < 0 ) && ( zDeltaY < 0 ))
        {
            /* Q 3 */
            if( abs( zDeltaX ) < abs( zDeltaY ))
            {
                psCurAnim->dx = 0 - psCurAnim->dx;
            }
            else
            {
                psCurAnim->dx = 0 - psCurAnim->dx;
            }
        }
        else if(( zDeltaX > 0 ) && ( zDeltaY < 0 ))
        {
            /* Q 4 */
            if( abs( zDeltaX ) < abs( zDeltaY ))
            {
                psCurAnim->dy = 0 - psCurAnim->dy;
            }
            else
            {
                psCurAnim->dx = 0 - psCurAnim->dx;
            }
        }
    }
}

/*-------------------------------------------------------------------
Name......... gm_DummyAnimation()
Author....... Rod Boyce
Date......... 01/10/04
Description.. Dummy function to for Heber animation
Parameters... 1) psAnim - Pointer to the animation structure
Return....... None
History...... 01/10/04 - First Pass  - RB
-------------------------------------------------------------------*/

static void gm_DummyAnimation( ANIMATION *sAnim )
{

}

/*-------------------------------------------------------------------
Name......... gm_IsBallAtEdge()
Author....... Rod Boyce
Date......... 01/10/04
Description.. This function is used to detect the edge of the screen
............. This function is called by the Calypso video driver to
............. when updating the position of the animation defined by
............. the pointer sAnim.  This function is used to make the
............. green and red ball move around the screen.
Parameters... 1) psAnim - pointer to the animation to perform this
.............    operation on
Return....... None
History...... 01/10/04 - First Pass  - RB
-------------------------------------------------------------------*/

static void gm_IsBallAtEdge( ANIMATION *psAnim )
{
    WORD zwScreenX;
    WORD zwScreenY;
    BYTE qChangeX = 0;
    BYTE qChangeY = 0;
    ANIMATION *psOthAnim;

    if( psAnim == &sRedBall )
    {
        psOthAnim = &sGreenBall;
    }
    else
    {
        psOthAnim = &sRedBall;
    }

    zwScreenX = REAL_HSIZE - psAnim->width;
    zwScreenY = REAL_VSIZE - psAnim->height;

    /* Check for Collision here */
    gm_CheckForCollision( psAnim, psOthAnim );

    /* Update x y position of animation here */
    psAnim->x += psAnim->dx;
    psAnim->y += psAnim->dy;

    if( psAnim->x < 0 )
    {
        qLeftHit = 1;
        psAnim->x = 0;
        qChangeX = 1;
    }
    if( psAnim->x > zwScreenX )
    {
        qRightHit = 1;
        psAnim->x = zwScreenX;
        qChangeX = 1;
    }
    if( psAnim->y < 0 )
    {
        psAnim->y = 0;
        qChangeY = 1;
    }
    if( psAnim->y > zwScreenY )
    {
        psAnim->y = zwScreenY;
        qChangeY = 1;
    }

    if( 0 != qChangeX )
    {
        psAnim->dx = 0 - psAnim->dx;
    }
    if( 0 != qChangeY )
    {
        psAnim->dy = 0 - psAnim->dy;
    }
}

/*-------------------------------------------------------------------
Name......... gm_InitialiseSound()
Author....... Rod Boyce
Date......... 01/10/04
Description.. Load the Game sounds into the wave info structures
............. This function will load the game sound files into the
............. wave file structures so that when a sound event occurs
............. it will be played.
Parameters... None
Return....... None
History...... 01/10/04 - First Pass  - RB
-------------------------------------------------------------------*/

static void gm_InitialiseSound( void )
{
	BYTE str[256];
    InitStreamedAudio( &ATADevice );

    if ( InitWavFile( apcSoundFiles[ 0 ], &sLoopSound ))
    {
		sprintf(str, "Failed to load %s\r",	apcSoundFiles[0]);
		TxString(pSerialDevice, 0, str );
    }
    else
    {
		sprintf(str, "%s loaded\r",	apcSoundFiles[0]);
		TxString(pSerialDevice, 0, str );
    }

    if ( InitWavFile( apcSoundFiles[ 1 ], &sLeftBounce ))
    {
		sprintf(str, "Failed to load %s\r",	apcSoundFiles[1]);
		TxString(pSerialDevice, 0, str );
    }
    else
    {
		sprintf(str, "%s loaded\r",	apcSoundFiles[1]);
		TxString(pSerialDevice, 0, str );
    }

    if ( InitWavFile( apcSoundFiles[ 2 ], &sRightBounce ))
    {
		sprintf(str, "Failed to load %s\r",	apcSoundFiles[2]);
		TxString(pSerialDevice, 0, str );
    }
    else
    {
		sprintf(str, "%s loaded\r",	apcSoundFiles[2]);
		TxString(pSerialDevice, 0, str );
    }

    if ( InitWavFile( apcSoundFiles[ 3 ], &sCollision ))
    {
		sprintf(str, "Failed to load %s\r",	apcSoundFiles[3]);
		TxString(pSerialDevice, 0, str );
	}
    else
    {
		sprintf(str, "%s loaded\r",	apcSoundFiles[3]);
		TxString(pSerialDevice, 0, str );
    }

    SetAudioVolume( AUDIO_LEFT0, (BYTE)200 );
    SetAudioVolume( AUDIO_RIGHT0, (BYTE)200 );
}
